import unittest

# from panther_analysis_tool.backend.public_api_client import PublicAPIClient, PublicAPIClientOptions
# from panther_analysis_tool.util import get_current_version


class DarGraphQL(unittest.TestCase):
    pass
    # def test_version(self):
    #     print(get_current_version())
    #
    # def test_behavior(self):
    #     cli = PublicAPIClient(PublicAPIClientOptions(
    #         host="https://nt7exr21xa.execute-api.us-east-2.amazonaws.com/v1/public/graphql",
    #         token="4rrhj1A4rZ6Zg4B4SXHZdaEF60Ex3oC5auMj187X",
    #         user_id="00000000-0000-4000-8000-000000000000"
    #     ))
    #
    #     self.assertFalse(cli.supports_async_uploads())
